package PayServer

import (
	"encoding/json"
	"game/base/log"
	"game/base/platApi"
	"game/base/redis"
	"game/base/util"
	"game/config"
	"game/excel"
	"net/http"
	"strconv"
	"strings"
)

type HuaweiPreOrderRet struct {
	Amount         string `json:"amount"`
	ApplicationID  string `json:"applicationID"`
	Url            string `json:"url"`
	ProductDesc    string `json:"productDesc"`
	ProductName    string `json:"productName"`
	ServiceCatalog string `json:"serviceCatalog"`
	MerchantId     string `json:"merchantId"`
	MerchantName   string `json:"merchantName"`
	Sign           string `json:"sign"`
	RequestId      string `json:"requestId"`
	Urlver         string `json:"urlver"`
	SdkChannel     int    `json:"sdkChannel"`
	PublicKey      string `json:"publicKey"`
	ExtReserved    string `json:"extReserved"`
}

func HuaweiPreOrder(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		return
	}
	ret := &HuaweiPreOrderRet{}
	defer func() {
		data, _ := json.Marshal(ret)
		w.Write(data)
	}()

	r.ParseForm()
	//account := r.PostFormValue("account")
	produce := r.PostFormValue("produce")
	//session := r.PostFormValue("session")
	uid := r.PostFormValue("uid")
	sid := r.PostFormValue("server")
	log.Debugf("HuaweiPreOrder Param %+v", r.Form)

	if !redis.HExists("game_info", sid) {
		log.Error("not Found server ", r.Form.Encode())
		return
	}

	info := excel.GetPlatformIdById(util.StringToUint32(produce))
	ret.Amount = strconv.FormatUint(uint64(info.Money), 10) + ".00"
	ret.ApplicationID = config.HuaweiAppId
	ret.Url = config.CmConfig.AppPayUrl + "/huaweiCallback"
	ret.ProductDesc = info.Name
	ret.ProductName = info.Name
	billNo, _ := util.NextId()
	ret.RequestId = util.Int64ToString(billNo)
	ret.ServiceCatalog = "X6"
	ret.MerchantId = config.HuaweiPayId
	ret.MerchantName = "海南霍比网络科技有限公司"
	ret.Urlver = "2"
	ret.SdkChannel = 3
	ret.PublicKey = config.HuaweiPublicKey
	ret.ExtReserved = sid + "-" + produce + "-" + uid

	if config.CmConfig.IsHuaweiPayDebug {
		ret.Amount = "0.01"
	}

	m := make(map[string]interface{})
	m["amount"] = ret.Amount
	m["applicationID"] = ret.ApplicationID
	m["url"] = ret.Url
	m["productDesc"] = ret.ProductDesc
	m["productName"] = ret.ProductName
	m["requestId"] = ret.RequestId
	m["merchantId"] = ret.MerchantId
	m["urlver"] = ret.Urlver
	m["sdkChannel"] = ret.SdkChannel

	var err error
	ret.Sign, err = platApi.GenSign(m, config.HuaweiPrivateKey)
	if err != nil {
		log.Error("GenSin Fail err:", err)
	}
	log.Info("HuaweiPreOrder ret:", ret)
}

type HuaweiCallbackRet struct {
	Result int `json:"result"`
}

func (srv *PayServer) HuaweiPayCallBack(w http.ResponseWriter, r *http.Request) {
	ret := &HuaweiCallbackRet{}
	defer func() {
		data, _ := json.Marshal(ret)
		w.Write(data)
		log.Info("HuaweiPayCallBack result:", ret.Result)
	}()

	if r.Method != "POST" {
		return
	}

	r.ParseForm()
	result := r.FormValue("result")
	userName := r.FormValue("userName")
	productName := r.FormValue("productName")
	payType := r.FormValue("payType")
	amount := r.FormValue("amount")
	currency := r.FormValue("currency")
	orderId := r.FormValue("orderId")
	notifyTime := r.FormValue("notifyTime")
	requestId := r.FormValue("requestId")
	bankId := r.FormValue("bankId")
	orderTime := r.FormValue("orderTime")
	tradeTime := r.FormValue("tradeTime")
	accessMode := r.FormValue("accessMode")
	spending := r.FormValue("spending")
	timeOffset := r.FormValue("timeOffset")
	extReserved := r.FormValue("extReserved")
	sysReserved := r.FormValue("sysReserved")

	signType := r.FormValue("signType")
	sign := r.FormValue("sign")

	//log.Info("HuaweiPayCallBack result:", result, " userName:", userName, " productName:", productName, " payType:", payType,
	//	" amount:", amount, " currency:", currency, " orderId:", orderId, " notifyTime:", notifyTime, " requestId:", requestId,
	//	" bankId:", bankId, " orderTime:", orderTime, " tradeTime:", tradeTime, " accessMode:", accessMode, " spending:", spending,
	//	" timeOffset:", timeOffset, " extReserved:", extReserved, " sysReserved:", sysReserved, " signType:", signType, " sign:", sign)

	m := make(map[string]interface{})
	m["result"] = result
	m["userName"] = userName
	m["productName"] = productName
	m["payType"] = payType
	m["amount"] = amount
	m["currency"] = currency
	m["orderId"] = orderId
	m["notifyTime"] = notifyTime
	m["requestId"] = requestId
	m["bankId"] = bankId
	m["orderTime"] = orderTime
	m["tradeTime"] = tradeTime
	m["accessMode"] = accessMode
	m["spending"] = spending
	m["extReserved"] = extReserved
	m["sysReserved"] = sysReserved
	m["timeOffset"] = timeOffset

	if err := platApi.CheckSign(m, sign, config.HuaweiPublicKey, signType); err != nil {
		ret.Result = 1
		log.Error("CheckSign Fail err:", err)
		return
	}

	extInfo := strings.Split(extReserved, "-")
	if len(extInfo) != 3 {
		ret.Result = 4
		return
	}
	sid := util.StringToInt(extInfo[0])
	produce := util.StringToUint32(extInfo[1])
	uid := util.StringToUint64(extInfo[2])

	info := excel.GetPlatformIdById(produce)
	msg := &RechargeCommonC2SMsg{
		RoleId:     uid,
		Rmb:        float32(info.Money),
		ChargeYb:   info.Money * 10,
		GiftYb:     0,
		OrderId:    r.FormValue("cpOrderId"),
		POrderId:   r.FormValue("orderId"),
		PlatformIp: r.Header.Get("X-Real_IP"),
		AttType:    info.Type,
		AttValue:   info.Index,
	}
	code, err := srv.DeliverGood(sid, msg)
	if code == 1 || code == 1002 {
		ret.Result = 0
	} else {
		ret.Result = 3
	}

	log.Info("HuaweiCallback DeliverRecv sid:", sid, " Role:", uid, " OrderId:", msg.OrderId, " POrderId:", msg.POrderId,
		" Money:", info.Money, " produce:", produce, " Result:", ret.Result, " code:", code, " err:", err)
	return
}
