package PayServer

import (
	"encoding/json"
	"fmt"
	"game/base/log"
	"game/base/platApi"
	"game/base/util"
	"game/config"
	"game/excel"
	"net/http"
	"strings"
	"time"
)

func OppoPreOrder(w http.ResponseWriter, r *http.Request) {
	var data []byte
	defer func() {
		w.Write(data)
	}()

	r.ParseForm()
	openid := r.PostFormValue("account")
	produce := r.PostFormValue("produce")
	engineVersion := r.PostFormValue("engineVersion")
	sid := r.PostFormValue("server")
	uid := r.PostFormValue("uid")
	attach := fmt.Sprintf("%d|%s|%s|%s", time.Now().Unix(), sid, produce, uid)
	info := excel.GetPlatformIdById(util.StringToUint32(produce))
	if info == nil {
		log.Error("Wrong param ", produce)
		return
	}
	fee := int64(info.Money * 100)
	if config.CmConfig.Debug {
		fee = 1
	}
	ret, err := platApi.OppoReqOrder(
		config.OppoAppID,
		config.OppoAppKey,
		openid,
		info.Name,
		info.Name,
		"1.0.1",
		engineVersion,
		1,
		fee,
		config.CmConfig.AppPayUrl+"/oppoCallback",
		config.OppoPrivateKey,
		attach)
	if err != nil {
		log.Errorf("PreOrder Error %s", err)
		return
	}

	data, _ = json.Marshal(ret)
	log.Info("OppoPreOrder ret:", ret)
}

func (srv *PayServer) OppoPayCallBack(w http.ResponseWriter, r *http.Request) {
	var ret string
	defer func() {
		log.Infof("OppoPayCallBack %s", ret)
		w.Write([]byte(ret))
	}()

	if r.Method != "POST" {
		return
	}

	r.ParseForm()
	log.Debug("OppoCallBack ", r.Form)
	err := platApi.OppoCallBackVerify(r.PostForm, config.OppoPublicKey)
	if err != nil {
		ret = "result=FAIL&resultMsg=参数验证失败" + err.Error()
		return
	}

	ret = "result=OK&resultMsg=成功"
	attach := strings.Split(r.FormValue("attach"), "|")
	sid := util.StringToInt(attach[1])
	produce := util.StringToUint32(attach[2])
	info := excel.GetPlatformIdById(produce)
	msg := &RechargeCommonC2SMsg{
		RoleId:     util.StringToUint64(attach[3]),
		Rmb:        float32(info.Money),
		ChargeYb:   info.Money * 10,
		GiftYb:     0,
		OrderId:    r.FormValue("partnerOrder"),
		POrderId:   r.FormValue("notifyId"),
		PlatformIp: r.Header.Get("X-Real_IP"),
		AttType:    info.Type,
		AttValue:   info.Index,
	}
	c, m := srv.DeliverGood(sid, msg)
	if c != 1 && c != 1002 {
		ret = "result=FAIL&resultMsg=" + m
	}
	log.Infof("OppoCallBack DeliverRecv sid:%d Role:%s cpOrder:%s order:%s Rmb:%d product:%d ret %d",
		sid, attach[3], msg.OrderId, msg.POrderId, info.Money, produce, c)
	return
}
