package PayServer

import (
	"context"
	"encoding/json"
	"fmt"
	"game/base/log"
	"game/base/redis"
	"io/ioutil"
	"net/http"
	"net/url"
	"os"
	"os/signal"
	"syscall"
	"time"
)

// Server 服务器
type PayServer struct {
	addr string
}

func (srv *PayServer) SetAddr(addr string) {
	srv.addr = addr
}

// Run 运行
func (srv *PayServer) Run() {
	//开始access Token服务
	//platApi.StartAccessTokenMgr()

	http.HandleFunc("/qqPreOrder", QQPreOrder)
	http.HandleFunc("/qqCallback", srv.QQPayCallBack)
	http.HandleFunc("/oppoPreOrder", OppoPreOrder)
	http.HandleFunc("/oppoCallback", srv.OppoPayCallBack)
	http.HandleFunc("/xiaomiCallback", srv.XiaomiPayCallBack)
	http.HandleFunc("/xiaomiPreOrder", XiaomiPreOrder)
	http.HandleFunc("/hGamePreOrder", HGamePreOrder)
	http.HandleFunc("/hGameCallBack", srv.HGamePayCallBack)
	http.HandleFunc("/hGameLogin", HGameLogin)
	http.HandleFunc("/checkWDReward", srv.CheckSendWeiDuanReward)
	http.HandleFunc("/WDReward", srv.SendWeiDuanReward)
	http.HandleFunc("/checkAuthReward", srv.CheckAuthReward)
	http.HandleFunc("/AuthReward", srv.AuthReward)
	http.HandleFunc("/huaweiCallback", srv.HuaweiPayCallBack)
	http.HandleFunc("/huaweiPreOrder", HuaweiPreOrder)
	http.HandleFunc("/checkRegisterReward", srv.CheckRegisterReward)
	http.HandleFunc("/RegisterReward", srv.RegisterReward)

	server := http.Server{Addr: srv.addr, Handler: http.DefaultServeMux}
	go func() {
		err := server.ListenAndServe()
		if err != nil && err != http.ErrServerClosed {
			panic(err)
		}
	}()
	log.Info("PayServer Start! addr:", srv.addr)

	c := make(chan os.Signal, 1)
	signal.Notify(c, syscall.SIGINT, syscall.SIGQUIT, syscall.SIGTERM, syscall.SIGKILL, os.Interrupt, os.Kill)
	<-c

	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	defer cancel()
	server.Shutdown(ctx)
	log.Info("PayServer Stoped!")
}

func (srv *PayServer) DeliverGood(sid int, msg *RechargeCommonC2SMsg) (code int, err string) {
	recv, e := srv.SendMsgToGame(sid, "charge/commc", msg, "")
	if e != nil {
		return 111, e.Error()
	}
	ret := &RechargeCommonS2CMsg{}
	json.Unmarshal(recv, ret)
	code = ret.Code
	return
}

type RechargeCommonC2SMsg struct {
	UserId     string  `json:"userId"`
	AreaId     int     `json:"areaId"`
	WorldId    int     `json:"worldId"`
	Pid        string  `json:"pid"`
	OrderId    string  `json:"orderId"`
	POrderId   string  `json:"pOrderId"`
	RoleId     uint64  `json:"roleId"`
	PlatformIp string  `json:"platformIp"`
	Rmb        float32 `json:"rmb"`
	ChargeYb   int     `json:"chargeYb"`
	GiftYb     int     `json:"giftYb"`
	AttData    string  `json:"attData"`
	AttType    int     `json:"attType"`
	AttValue   int     `json:"attValue"`
}
type RechargeCommonS2CMsg struct {
	Code    int     `json:"code"`
	OrderId string  `json:"orderId"`
	Rmb     float32 `json:"rmb"`
	Yb      int     `json:"yb"`
	LoginIp string  `json:"loginIp"`
}

type ServerInfo struct {
	AreaId        int    `json:"areaId"`
	CombineTime   int    `json:"combineTime"`
	ExternalIp    string `json:"externalIp "`
	FollowerId    int    `json:"followerId"`
	GameRedisPort int    `json:"gameRedisPort"`
	GameRedsiHost string `json:"gameRedsiHost"`
	Hidden        bool   `json:"hidden"`
	InnerIp       string `json:"innerIp"`
	OpenTime      int    `json:"openTime"`
	Pid           string `json:"pid"`
	Status        int    `json:"status"`
	TcpPort       int    `json:"tcpPort"`
	WebPort       int    `json:"webPort"`
	WorldId       int    `json:"worldId"`
	WorldName     string `json:"worldName"`
}

func (srv *PayServer) SendMsgToGame(sid int, urlPath string, msg interface{}, name string) (respData []byte, e error) {
	info := redis.HGet("game_info", sid)
	if info == "" {
		log.Debug("SendMsgToGame info=nil")
		return
	}
	server := &ServerInfo{}
	e = json.Unmarshal([]byte(info), server)
	if e != nil {
		log.Debug("SendMsgToGame e:", e)
		return
	}

	var data []byte
	data, e = json.Marshal(msg)
	if e != nil {
		log.Debug("SendMsgToGame e:", e)
		return
	}
	sUrl := fmt.Sprintf("http://%s:%d/api/%s?parameter=%s", server.InnerIp, server.WebPort, urlPath, url.QueryEscape(string(data)))
	ss, _ := url.QueryUnescape(sUrl)
	log.Info("SendMsgToGame Url ", ss)

	var resp *http.Response
	resp, e = http.Get(sUrl)
	if e != nil {
		log.Error("SendMsgToGame Get fail err:", e)
		return
	}
	respData, e = ioutil.ReadAll(resp.Body)
	if e != nil {
		log.Error("SendMsgToGame ReadAll fail err:", e)
		return
	}
	resp.Body.Close()

	rr := &MailSenderS2CMsg{}
	if e = json.Unmarshal(respData, rr); e != nil {
		log.Error("SendMsgToGame Unmarshal fail err:", e)
		return
	}
	if rr.Code == 1 {
		return
	}
	log.Info("SendMail Fail msg:", msg, " sid:", sid, " name:", name, " rr:", rr)
	return
}
