package PayServer

import (
	"encoding/json"
	"fmt"
	"game/base/log"
	"game/base/platApi"
	"game/base/redis"
	"game/base/util"
	"game/config"
	"game/excel"
	"net/http"
	"net/url"
	"strings"
	"time"
)

type XiaomiPreOrderRet struct {
	AppID        string                         `json:"appId"`
	AppAccountID string                         `json:"appAccountId"`
	Session      string                         `json:"session"`
	CpOrderID    string                         `json:"cpOrderId"`
	CpUserInfo   string                         `json:"cpUserInfo"`
	DisplayName  string                         `json:"displayName"`
	FeeValue     int                            `json:"feeValue"`
	Sign         string                         `json:"sign"`
	SessionCheck *platApi.XiaomiCheckSessionRet `json:"sessionCheck"`
}

func XiaomiPreOrder(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		return
	}
	ret := &XiaomiPreOrderRet{}
	defer func() {
		data, _ := json.Marshal(ret)
		w.Write(data)
	}()

	r.ParseForm()
	account := r.PostFormValue("account")
	produce := r.PostFormValue("produce")
	session := r.PostFormValue("session")
	uid := r.PostFormValue("uid")
	sid := r.PostFormValue("server")
	log.Debugf("XiaomiPreOrder Param %+v", r.Form)
	c := platApi.XiaomiCheckSession(config.XiaomiAppID, config.XiaomiAppSecret, account, session)
	if c == nil || c.Errcode != 200 {
		ret.SessionCheck = c
		if c != nil {
			log.Errorf("xiaomi session check error %d | %s ", c.Errcode, c.ErrMsg)
		}
		return
	}
	if !redis.HExists("game_info", sid) {
		log.Error("not Found server ", r.Form.Encode())
		return
	}
	info := excel.GetPlatformIdById(util.StringToUint32(produce))
	ret.AppAccountID = account
	ret.AppID = config.XiaomiAppID
	ret.Session = session
	billNo, _ := util.NextId()
	ret.CpOrderID = util.Int64ToString(billNo)
	ret.CpUserInfo = fmt.Sprintf("%d|%s|%s|%s", time.Now().Unix(), sid, produce, uid)
	ret.DisplayName = info.Name
	ret.FeeValue = info.Money * 100
	if config.CmConfig.Debug {
		ret.FeeValue = 1
	}
	values := url.Values{}
	values.Add("appId", ret.AppID)
	values.Add("appAccountId", ret.AppAccountID)
	values.Add("session", ret.Session)
	values.Add("cpOrderId", ret.CpOrderID)
	values.Add("cpUserInfo", ret.CpUserInfo)
	values.Add("displayName", ret.DisplayName)
	values.Add("feeValue", util.IntToString(ret.FeeValue))
	ret.Sign = platApi.XiaomiSign(values, config.XiaomiAppPayKey)
	log.Info("XiaomiPreOrder ret:", ret)
}

type XiaomiCallbackRet struct {
	Errcode int    `json:"errcode"`
	ErrMsg  string `json:"errMsg"`
}

func (srv *PayServer) XiaomiPayCallBack(w http.ResponseWriter, r *http.Request) {
	ret := &XiaomiCallbackRet{}
	defer func() {
		data, _ := json.Marshal(ret)
		w.Write(data)
	}()

	r.ParseForm()
	if !platApi.XiaomiCallbackCheck(r.Form, config.XiaomiAppSecret) {
		ret.Errcode = 100
		ret.ErrMsg = "签名错误"
		return
	}
	cpUserInfo := strings.Split(r.FormValue("cpUserInfo"), "|")
	sid := util.StringToInt(cpUserInfo[1])
	produce := util.StringToUint32(cpUserInfo[2])
	info := excel.GetPlatformIdById(produce)
	msg := &RechargeCommonC2SMsg{
		RoleId:     util.StringToUint64(cpUserInfo[3]),
		Rmb:        float32(info.Money),
		ChargeYb:   info.Money * 10,
		GiftYb:     0,
		OrderId:    r.FormValue("cpOrderId"),
		POrderId:   r.FormValue("orderId"),
		PlatformIp: r.Header.Get("X-Real_IP"),
		AttType:    info.Type,
		AttValue:   info.Index,
	}
	ret.Errcode, ret.ErrMsg = srv.DeliverGood(sid, msg)
	log.Infof("XiaomiCallback DeliverRecv sid:%d Role:%s cpOrder:%s order:%s Rmb:%d product:%d ret %d",
		sid, cpUserInfo[3], msg.OrderId, msg.POrderId, info.Money, produce, ret.Errcode)
	if ret.Errcode == 1 || ret.Errcode == 1002 {
		ret.Errcode = 200
		ret.ErrMsg = "success"
	}
	return
}
