package log

import (
	rotatelogs "github.com/lestrrat-go/file-rotatelogs"
	"github.com/rifflock/lfshook"
	"github.com/sirupsen/logrus"
	"os"
	"strings"
	"time"
)

// 日志分割hook
func initLfsHook(filePath, fileName, version string) (logrus.Hook, error) {
	filePath = filePath + version + "/"
	fileName = strings.ToLower(fileName)

	os.MkdirAll(filePath, os.ModePerm)
	writer, err := rotatelogs.New(
		filePath+fileName+".log.%Y%m%d",
		//rotatelogs.WithLinkName(filePath+fileName), //为最新的日志建立软连接,以方便找到当前日志文件
		rotatelogs.WithRotationTime(time.Hour*24), //设置日志分割的时间,这里设置为24小时分割一次
		rotatelogs.WithMaxAge(time.Hour*24*30),    //设置文件清理前的最长保存时间, WithMaxAge和WithRotationCount二者只能设置一个,
		//rotatelogs.WithRotationCount(30),         //设置文件清理前最多保存的个数.
	)
	if err != nil {
		logrus.Error(err)
		return nil, err
	}

	return lfshook.NewHook(lfshook.WriterMap{
		logrus.InfoLevel:  writer,
		logrus.WarnLevel:  writer,
		logrus.ErrorLevel: writer,
		logrus.FatalLevel: writer,
		logrus.PanicLevel: writer,
		//}, &logrus.JSONFormatter{TimestampFormat: "2006-01-02 15:04:05"})
	}, &logrus.TextFormatter{TimestampFormat: "2006-01-02 15:04:05"}), nil
}
