package platApi

import (
	"crypto"
	"crypto/rand"
	"crypto/rsa"
	"crypto/sha1"
	"crypto/sha256"
	"crypto/x509"
	"encoding/base64"
	"fmt"
	"game/base/log"
	"sort"
)

func GenSign(data map[string]interface{}, private string) (string, error) {
	//STEP 1, 对key进行升序排序.
	sorted_keys := make([]string, 0)
	for k := range data {
		sorted_keys = append(sorted_keys, k)
	}
	sort.Strings(sorted_keys)

	//STEP2, 对key=value的键值对用&连接起来，略过空值
	var signStrings string
	for i, k := range sorted_keys {
		value := fmt.Sprintf("%v", data[k])
		if value != "" {
			if i != (len(sorted_keys) - 1) {
				signStrings = signStrings + k + "=" + value + "&"
			} else {
				signStrings = signStrings + k + "=" + value //最后一个不加此符号
			}
		}
	}
	keyByts, err := base64.StdEncoding.DecodeString(private)
	if err != nil {
		log.Error(err)
		return "", err
	}

	privateKey, err := x509.ParsePKCS8PrivateKey(keyByts)
	if err != nil {
		log.Error("ParsePKCS8PrivateKey err", err)
		return "", err
	}
	h := sha256.New()
	h.Write([]byte(signStrings))
	signature, err := rsa.SignPKCS1v15(rand.Reader, privateKey.(*rsa.PrivateKey), crypto.SHA256, h.Sum(nil))
	if err != nil {
		log.Errorf("Error from signing: %s\n", err)
		return "", err
	}
	return base64.StdEncoding.EncodeToString(signature), nil
}

func CheckSign(data map[string]interface{}, signData, pubKey, signType string) error {
	//STEP 1, 对key进行升序排序.
	sorted_keys := make([]string, 0)
	for k := range data {
		sorted_keys = append(sorted_keys, k)
	}
	sort.Strings(sorted_keys)

	//STEP2, 对key=value的键值对用&连接起来，略过空值
	var signStrings string
	for i, k := range sorted_keys {
		value := fmt.Sprintf("%v", data[k])
		if value != "" {
			if i != (len(sorted_keys) - 1) {
				signStrings = signStrings + k + "=" + value + "&"
			} else {
				signStrings = signStrings + k + "=" + value //最后一个不加此符号
			}
		}
	}

	sign, err := base64.StdEncoding.DecodeString(signData)
	if err != nil {
		return err
	}
	public, _ := base64.StdEncoding.DecodeString(pubKey)
	pub, err := x509.ParsePKIXPublicKey(public)
	if err != nil {
		return err
	}

	if signType != "" {
		hash := sha256.New()
		hash.Write([]byte(signStrings))
		err = rsa.VerifyPKCS1v15(pub.(*rsa.PublicKey), crypto.SHA256, hash.Sum(nil), sign)
	} else {
		hash := sha1.New()
		hash.Write([]byte(signStrings))
		err = rsa.VerifyPKCS1v15(pub.(*rsa.PublicKey), crypto.SHA1, hash.Sum(nil), sign)
	}
	return err
}
