package platApi

import (
	"encoding/json"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"

	"game/base/log"
)

type WXTokenRes struct {
	AccessToken string `json:"access_token"`
	ExpiresIn   uint32 `json:"expires_in"`
	ErrCode     uint32 `json:"errcode"`
	ErrMsg      string `json:"errmsg"`
}

func WXAccessToken(appID string, secret string) (key string, expire uint32, err error) {
	url1 := "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + appID + "&secret=" + secret
	resp, err := http.Get(url1)
	if err != nil {
		return
	}
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return
	}
	resp.Body.Close()
	ret := &WXTokenRes{}
	err = json.Unmarshal(body, &ret)
	if err != nil || ret.ErrCode != 0 {
		return
	}
	key = ret.AccessToken
	expire = ret.ExpiresIn
	return
}

func WXAppQRCode(token string, scene, page string) (float64, []byte) {
	hUrl := "https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token=" + token
	values := map[string]string{}
	values["scene"] = scene
	values["page"] = page
	d, e := json.Marshal(values)
	resp, e := http.Post(hUrl, "application/json; charset=utf-8", strings.NewReader(string(d)))
	if e != nil {
		log.Error("GetQRCode Error ", e)
		return 1, nil
	}
	defer resp.Body.Close()
	body, e := ioutil.ReadAll(resp.Body)
	if e != nil {
		return 1, nil
	}
	ret := map[string]interface{}{}
	json.Unmarshal(body, &ret)
	if d, ok := ret["errcode"]; ok {
		log.Error("GetQRCode Error ", ret)
		return d.(float64), nil
	}
	return 0, body
}

type WXSessionRet struct {
	Openid     string `json:"openid"`
	SessionKey string `json:"session_key"`
	UinionId   string `json:"uinionid"`
	Errcode    uint   `json:"errcode"`
	ErrMsg     string `json:"errMsg"`
}

func WXCode2Session(code string, appID string, secret string) *WXSessionRet {
	client := http.Client{}
	param := url.Values{}
	param.Set("appid", appID)
	param.Set("secret", secret)
	param.Set("js_code", code)
	param.Set("grant_type", "authorization_code")
	urlStr := "https://api.weixin.qq.com/sns/jscode2session?" + param.Encode()
	res, err := client.Get(urlStr)

	if err != nil {
		log.Error("Get Access Token Error ", urlStr, err)
		return nil
	}

	body, bodyErr := ioutil.ReadAll(res.Body)
	if bodyErr != nil {
		log.Error("Get Access Token Error ", bodyErr)
		return nil
	}
	res.Body.Close()
	ret := &WXSessionRet{}
	e := json.Unmarshal(body, ret)
	if e != nil {
		log.Error("Get Access Token Error ", e)
		return nil
	}
	if ret.Errcode != 0 {
		log.Error("Get Access Token Error ", ret.Errcode, ret.ErrMsg)
		return nil
	}
	return ret
}
