package platApi

import (
	"crypto/hmac"
	"crypto/sha1"
	"encoding/hex"
	"encoding/json"
	"game/base/log"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
)

type XiaomiCheckSessionRet struct {
	Errcode int    `json:"errcode"`
	ErrMsg  string `json:"errMsg"`
	Adult   int    `json:"adult"`
}

func XiaomiCheckSession(appID string, appSecret string, account string, session string) *XiaomiCheckSessionRet {
	client := http.Client{}
	param := url.Values{}
	param.Add("appId", appID)
	param.Add("session", session)
	param.Add("uid", account)
	param.Add("signature", XiaomiSign(param, appSecret))
	urlStr := "https://mis.migc.xiaomi.com/api/biz/service/loginvalidate"
	res, err := client.Post(urlStr, "application/x-www-form-urlencoded", strings.NewReader(param.Encode()))
	if err != nil {
		log.Error("XiaomiCheckSession Error ", urlStr, err)
		return nil
	}

	body, bodyErr := ioutil.ReadAll(res.Body)
	if bodyErr != nil {
		log.Error("XiaomiCheckSession Error ", bodyErr)
		return nil
	}
	res.Body.Close()
	ret := &XiaomiCheckSessionRet{}
	e := json.Unmarshal(body, ret)
	if e != nil {
		log.Error("XiaomiCheckSession Error ", e)
		return nil
	}
	if ret.Errcode != 200 {
		log.Error("XiaomiCheckSession Error ", ret.Errcode, ret.ErrMsg)
	}
	return ret
}

func XiaomiSign(values url.Values, appSecret string) string {
	for k, v := range values {
		needDel := true
		for _, i := range v {
			if len(strings.TrimSpace(i)) != 0 {
				needDel = false
				break
			}
		}
		if needDel {
			values.Del(k)
		}
	}
	str := values.Encode()
	str, _ = url.QueryUnescape(str)
	h := hmac.New(sha1.New, []byte(appSecret))
	h.Write([]byte(str))
	data := h.Sum(nil)
	return hex.EncodeToString(data)
}

func XiaomiCallbackCheck(values url.Values, appSecret string) bool {
	sign := values.Get("signature")
	values.Del("signature")
	s := XiaomiSign(values, appSecret)
	return sign == s
}
