package excel

import (
	"encoding/json"
	"game/base/log"
	"io/ioutil"
	"os"
)

type PlatformIdTable struct {
	Id    uint32 `json:"id"`
	Index int    `json:"index"`
	Name  string `json:"name"`
	Type  int    `json:"type"`
	Money int    `json:"money"`
}

var platformIdMap map[uint32]*PlatformIdTable

func loadPlatformIdTable(fileName string) {
	data, err := ioutil.ReadFile(fileName)
	if err != nil && os.IsNotExist(err) {
		log.Errorf("Read Filename %s err %s: ", fileName, err)
		return
	}
	var temp []*PlatformIdTable
	err = json.Unmarshal(data, &temp)
	if err != nil {
		log.Error("Unmarshal err: ", err)
		return
	}
	loadLock.Lock()
	platformIdMap = map[uint32]*PlatformIdTable{}
	for _, i := range temp {
		platformIdMap[i.Id] = i
	}
	loadLock.Unlock()
}

func GetPlatformIdById(id uint32) *PlatformIdTable {
	loadLock.RLock()
	defer loadLock.RUnlock()
	return platformIdMap[id]
}
