package main

import (
	"flag"
	"game/PayServer"
	"game/base/log"
	"game/base/redis"
	"game/config"
	"game/excel"
	"gopkg.in/yaml.v2"
	"io/ioutil"
)

func main() {
	configFile := flag.String("config", "", "配置文件/绝对路径")
	flag.Parse()

	cd, e := ioutil.ReadFile(*configFile)
	if e != nil {
		log.Errorf("read config file error ", e)
		return
	}
	e = yaml.Unmarshal(cd, &config.CmConfig)
	if e != nil {
		log.Errorf("unmarshal config file error ", e)
		return
	}

	//logrus日志初始化
	log.Init(config.CmConfig.LogLevel)
	log.AddLfsHook(config.CmConfig.LogPath, "wjPayServer", "")
	excel.LoadExcel(config.CmConfig.ExcelPath)
	//redis初始化
	redis.SetConfig(config.CmConfig.Redis.Host, config.CmConfig.Redis.Password)
	srv := &PayServer.PayServer{}
	srv.SetAddr(config.CmConfig.Addr)
	srv.Run()
}
